Files
project_6/cccl_upstream/cudax/test/stf/local_stf/stackable2.cu
EngineX CI 56fd68e7dd [INFRA] Import NVIDIA/CCCL upstream as optimization reference library
CCCL (CUDA C++ Core Libraries) provides:
- CUB: device/block/warp-level GPU primitives (reduce, scan, sort, topk)
- Thrust: high-level parallel algorithms (transform_reduce, sort, scan)
- libcudacxx: CUDA C++ standard library (atomics, barriers, memory)
- cudax: experimental features (memory resources, allocators)
- Tuning policies: per-SM hardware-specific algorithm parameters

Competition optimization vectors mapped to CCCL:
- Output TPS (83% weight): warp_reduce, block_reduce, device_topk
- Input TPS (14% weight): device_scan, block_load, prefetch
- Cache TPS (3% weight): prefix caching strategy patterns
- Memory (0.9 util): pooled/cached/buddy allocators

Source: https://github.com/NVIDIA/cccl (shallow clone, HEAD only)
License: Apache-2.0
2026-07-30 09:35:51 +00:00

70 lines
1.9 KiB
Plaintext

//===----------------------------------------------------------------------===//
//
// Part of CUDASTF in CUDA C++ Core Libraries,
// under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
// SPDX-FileCopyrightText: Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES.
//
//===----------------------------------------------------------------------===//
/**
* @file
*
* @brief Experiment with local context nesting
*
*/
#include <cuda/experimental/stf.cuh>
using namespace cuda::experimental::stf;
int main()
{
stackable_ctx ctx;
int array[1024];
for (size_t i = 0; i < 1024; i++)
{
array[i] = 1 + i * i;
}
auto lA = ctx.logical_data(array).set_symbol("A");
// repeat : {tmp = a; tmp*=2; a+=tmp}
for (size_t iter = 0; iter < 10; iter++)
{
stackable_ctx::graph_scope_guard graph{ctx}; // RAII: automatic push/pop (lock_guard style)
auto tmp = ctx.logical_data(lA.shape()).set_symbol("tmp");
ctx.parallel_for(tmp.shape(), tmp.write(), lA.read())->*[] __device__(size_t i, auto tmp, auto a) {
tmp(i) = a(i);
};
ctx.parallel_for(tmp.shape(), tmp.rw())->*[] __device__(size_t i, auto tmp) {
tmp(i) *= 2;
};
ctx.parallel_for(lA.shape(), tmp.read(), lA.rw())->*[] __device__(size_t i, auto tmp, auto a) {
a(i) += tmp(i);
};
// ctx.pop() is called automatically when 'graph' goes out of scope
}
ctx.finalize();
// Verify the array has been updated correctly by the write-back mechanism
// Each iteration transforms each element: a_new = a_old + 2 * a_old = 3 * a_old
// Starting from array[i] = 1 + i*i, after 10 iterations:
// array[i] = 3^10 * (1 + i*i)
constexpr int pow3_10 = 59049; // 3^10
for (size_t i = 0; i < 1024; i++)
{
int expected = pow3_10 * (1 + static_cast<int>(i * i));
EXPECT(array[i] == expected);
}
}